module matching
  use types; use consts_dp
  use warnings_and_errors
  implicit none
  
  private
  public :: match, fixed_order_schemes

contains

  !----------------------------------------------------------------------
  function match(matching_scheme, &
       &            sigma, pt, resummation, resum_expanded, &
       &            sigmabar,dlumi_lumi,dlumi_lumi_expanded) result(res)
    character(len=*), intent(in) :: matching_scheme
    real(dp), intent(in) :: sigma(0:), pt(:), resummation(:), resum_expanded(1:,:), sigmabar(1:,:)
    real(dp), intent(in) :: dlumi_lumi(:), dlumi_lumi_expanded(:)
    real(dp)             :: res(size(pt))

    select case(trim(matching_scheme))
    case("a")
       res = matching_a(sigma, pt(:), resummation, resum_expanded, &
            &sigmabar(:,:),dlumi_lumi,dlumi_lumi_expanded)
    case("b")
       res = matching_b(sigma, pt(:), resummation, resum_expanded, &
            &sigmabar(:,:),dlumi_lumi,dlumi_lumi_expanded)
    case("c")
       res = matching_c(sigma, pt(:), resummation, resum_expanded, &
            &sigmabar(:,:),dlumi_lumi,dlumi_lumi_expanded)
    case default
       call wae_error("matching: unrecognised matching scheme "//matching_scheme)
    end select
  end function match
  
! --- new version
  function matching_a(sigma, pt, resummation, resum_expanded, sigmabar,dlumi_lumi,dlumi_lumi_expanded) result(res)
    real(dp), intent(in) :: sigma(0:), pt(:), resummation(:), resum_expanded(1:,:), sigmabar(1:,:)
    real(dp), intent(in) :: dlumi_lumi(:), dlumi_lumi_expanded(:)
    real(dp)             :: res(size(pt))
    !----------------------------------------------

    res = (resummation/sigma(0)/(1+dlumi_lumi))*&
         &(sigma(0)*(1+dlumi_lumi) + sigma(1)+sigmabar(1,:) + sigma(2)+sigmabar(2,:) &
         &                           -(resum_expanded(1,:) + resum_expanded(2,:))&
         &                           - (resum_expanded(1,:)/sigma(0)-dlumi_lumi_expanded)&
         &                           * (sigma(1)+sigmabar(1,:) - resum_expanded(1,:)))
  end function matching_a
  

  function matching_b(sigma, pt, resummation, resum_expanded, sigmabar,dlumi_lumi,dlumi_lumi_expanded) result(res)
    real(dp), intent(in) :: sigma(0:), pt(:), resummation(:), resum_expanded(1:,:), sigmabar(1:,:)
    real(dp), intent(in) :: dlumi_lumi(:), dlumi_lumi_expanded(:)
    real(dp)             :: res(size(pt))
    !----------------------------------------------

    res = (resummation/sigma(0)/(1+dlumi_lumi))*&
         &(sigma(0)*(1+dlumi_lumi) + sigma(1)+sigmabar(1,:) + sigmabar(2,:) &
         &                           -(resum_expanded(1,:) + resum_expanded(2,:))&
         &                           - (resum_expanded(1,:)/sigma(0)-dlumi_lumi_expanded)&
         &                           * (sigma(1)+sigmabar(1,:) - resum_expanded(1,:)))
  end function matching_b


  function matching_c(sigma, pt, resummation, resum_expanded, sigmabar,dlumi_lumi,dlumi_lumi_expanded) result(res)
    real(dp), intent(in) :: sigma(0:), pt(:), resummation(:), resum_expanded(1:,:), sigmabar(1:,:)
    real(dp), intent(in) :: dlumi_lumi(:), dlumi_lumi_expanded(:)
    real(dp)             :: res(size(pt))
    !----------------------------------------------
    real(dp) :: eff_NLL(size(pt)),eff_NLLi(2,size(pt)),eff(2,size(pt))

    ! This reduces to eff_NLL only for NLL resummation!
    eff_NLL = resummation/sigma(0)/(1+dlumi_lumi)
    eff_NLLi(1:2,:) = resum_expanded(1:2,:)/sigma(0)
    eff(1,:) = sigmabar(1,:)/sigma(0)
    eff(2,:) = sigmabar(2,:)/sigma(0)-sigma(1)*sigmabar(1,:)/sigma(0)**2

    res = eff_nll*(1+dlumi_lumi+eff(1,:)+eff(2,:)-(eff_NLLi(1,:)+eff_NLLi(2,:)) &
         &                           - (eff_NLLi(1,:)-dlumi_lumi_expanded)*(eff(1,:)-eff_NLLi(1,:)))
  end function matching_c


  function matching_logR(sigma, pt, resummation, resum_expanded, sigmabar) result(res)
    real(dp), intent(in) :: sigma(0:), pt(:), resummation(:), resum_expanded(1:,:), sigmabar(1:,:)
    real(dp)             :: res(size(pt))
    !----------------------------------------------
    real(dp) :: Z(size(pt))

    res = (resummation/sigma(0)+sigma(1))*exp((sigma(1)+sigmabar(1,:) -resum_expanded(1,:))/sigma(0)) &
         &                           *exp((sigmabar(2,:)-resum_expanded(2,:))/sigma(0) - &
         &((sigma(1)+sigmabar(1,:))**2 -resum_expanded(1,:)**2)/(two*sigma(0)**2))


  end function matching_logR


  !----------------------------------------------------------------------
  function fixed_order_schemes(scheme, sigma, sigmabar) result(res)
    character(len=*), intent(in) :: scheme
    real(dp), intent(in) :: sigma(0:), sigmabar(1:,:)
    real(dp)             :: res(size(sigmabar,dim=2)), norm

    select case(trim(scheme))
    case("a")
       norm = sum(sigma(:))
       res  = one + sum(sigmabar,dim=1)/norm
    case("b")
       norm = sum(sigma(0:1))
       res  = one + sum(sigmabar,dim=1)/norm
    case("c")
       res = 1 + sigmabar(1,:)/sigma(0) + sigmabar(2,:)/sigma(0)&
            &      - sigma(1)/sigma(0)**2*sigmabar(1,:)
    case default
       call wae_error("fixed_order_schemes: unrecognised fixed order scheme "//scheme)
    end select
  end function fixed_order_schemes
  
  
end module matching
